#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "main.h"
#include "keys.h"
#include "timer.h"
#include "Led_Display.h"

#ifndef EEMEM
// alle Textstellen EEMEM im Quellcode durch __attribute__ ... ersetzen
#define EEMEM  __attribute__ ((section (".eeprom")))
#endif

#define beep()	beeptimer = DEBOUNCE / 128

unsigned int ee_time[ 256 ] EEMEM;

/*************************************************************************************************************
*************************************************************************************************************/
void IO_Init( void )
{
//  OSCCAL = 185;
    IO_PORT |= OUTPUT | SUMMER;                                         // Licht & Summer Aus (low-aktiv)
    IO_DIR |= OUTPUT | SUMMER;                                          // Portrichtung setzen
    KEY_DIR &= ~ALL_KEYS;                                               // Portrichtung setzen
    KEY_PORT |= ALL_KEYS | ~ALL_KEYS;                                   // PullUp's Ein (auch unbenutzte Portpin's)
    DIGIT_PORT |= ALL_DIGITS;                                           // Port setzen
    DIGIT_DIR |= ALL_DIGITS;                                            // Portrichtung setzen

    DISP_PORT |= ~ALL_SEGMENTS;                                         // PullUp am unbenutzten Portpin ein

#ifdef SEGMENTDRIVER
    DISP_PORT &= ~ALL_SEGMENTS;                                         // mit Segmenttreiber
#else
    DISP_PORT |= ALL_SEGMENTS;                                          // ohne Segmenttreiber
#endif
 
    DISP_DIR |= ALL_SEGMENTS;                                           // Portrichtung setzen

    count_down = 0;
    i = 0;                                                              // Indexcounter 0 - 255 (unsigned char)

    while( eeprom_read_word( &ee_time[ i ] ) & 0x8000 )                 // letzten gespeicherten Wert suchen
    {
        if( ++i == 0 )
        {
            i = 255;
            break;                                                      // gesamtes EEprom nicht initialisiert
        }
    }

    preset_time = eeprom_read_word( &ee_time[ i ] );                    // gefundenen Wert holen

    if( preset_time > 5999 )                                            // falls nur Mll im EEprom steht...
    {
        preset_time = 0;
        eesave_cnt = 1;
    }

    time = preset_time;
}

/*************************************************************************************************************
******************** Programmbeginn **************************************************************************
*************************************************************************************************************/
int main( void )
{
    unsigned char acceleration = 0;
    unsigned int last_time = 0;

    IO_Init();

    unsigned int last_preset_time = preset_time;
    last_time = 6000;                                                   // unmglicher Wert...

    timer_init();
    sei();

    while( 1 )                                                          // Endlosschleife
    {
        if( !count_down && ( IO_PORT & OUTPUT ) )                       // Verriegelung
        {
        	if( !( key_state & KEY2 ) )                                 // ist KEY2 frei
        	{

                if( !( key_state & KEY1 ) )                             // ist entweder KEY1 frei (else!)
                {
                    if( get_key_press( KEY0 ) )                         // Time+
                    {
                        if( ( time + 1 ) < 6000 ) time++;

                        if( last_time != time ) beep();                 // hat sich an time was verndert?

                        preset_time = time;
                    }

                    if( get_key_rpt( KEY0 ) )                           // Time++
                    {
                        if( acceleration >= 60 )
                        {
                            if( ( time + 600 ) < 6000 ) time += 600;

                            else if( ( time + 60 ) < 6000 ) time += 60;

                            else if( ( time + 10 ) < 6000 ) time += 10;

                            else if( ( time + 1 ) < 6000 ) time++;
                        }

                        else if( acceleration >= 40 )
                        {
                            if( ( time + 60 ) < 6000 ) time += 60;

                            else if( ( time + 10 ) < 6000 ) time += 10;

                            else if( ( time + 1 ) < 6000 ) time++;
                        }

                        else if( acceleration >= 19 )
                        {
                            if( ( time + 10 ) < 6000 ) time += 10;

                            else if( ( time + 1 ) < 6000 ) time++;
                        }

                        else
                        {
                            if( ( time + 1 ) < 6000) time++;
                        }
    
                        if( acceleration < 255 ) acceleration++;

                        preset_time = time;
                    }
                }

                else if( !( key_state & KEY0 ) )                        // oder ist KEY0 frei
                {
                    if( get_key_press( KEY1 ) )                         // Time-
                    {
                        if( time ) time--;

                        if( last_time != time ) beep();                 // hat sich an time was verndert?

                        preset_time = time;
                    }
    
                    if( get_key_rpt( KEY1 ) )                           // Time--
                    {
                        if( acceleration >= 60 )
                        {
                            if( time >= 600 ) time -= 600;

                            else if( time >= 60 ) time -= 60;

                            else if( time >= 10 ) time -= 10;

                            else if( time ) time --;
                        }

                        else if( acceleration >= 40 )
                        {
                            if( time >= 60 ) time -= 60;

                            else if( time >= 10 ) time -= 10;

                            else if( time ) time --;
                        }

                        else if( acceleration >= 19 )
                        {
                            if( time >= 10 ) time -= 10;

                            else if( time ) time --;
                        }

                        else
                        {
                            if( time ) time--;
                        }
    
    
                        if( acceleration < 255 ) acceleration++;

                        preset_time = time;
                    }
                }

                else acceleration = 0;                                  // KEY0 && KEY1 waren nicht frei
            }
        }                                                               // end: if( !count_down && ( IO_PORT & LED_1 ) )

        if( !( key_state & ( KEY0 | KEY1 ) ) )
        {
            acceleration = 0;                                           // ist weder KEY0 noch KEY1 gedrckt...

            if( get_key_press( KEY2 ) )                                 // Start --- Stop
            {
                key_rpt = 0;
				beep();

                if( count_down ) count_down = 0;                        // luft der Countdown, anhalten

                else                                                    // luft der Countdown nicht
                {
                    if( time )                                          // time > 0 ?
                    {
                        compare = prescaler;                            // diesen Augenblick merken...
                        time--;                                         // einmal herunterzhlen (Start Countdown)
                        count_down = 1;                                 // Countdown ein
                        IO_PORT &= ~OUTPUT;                             // Licht Ein (low-aktiv)
                    }

                    else
                    {
                        if( IO_PORT & OUTPUT ) time = preset_time;      // time == 0, lade voreingestellten Wert
                    }
                }                                                       // end: else "luft der Countdown nicht"
            }                                                           // end: if( get_key_press( KEY2 ) )
        }                                                               // end: if( !( key_state & ( KEY0 | KEY1 ) ) )

        if( last_time != time )                                         // hat sich an time was verndert?
        {                                                               // Umrechnung in 4 einzelne Variablen
            minuten_zehner = ( time / 60 ) / 10;                        // zur Displaysteuerung
            minuten_einer = ( time / 60 ) % 10;
            sekunden_zehner = ( time % 60 ) / 10;
            sekunden_einer = ( time % 60 ) % 10;
            last_time = time;                                           // nderung bernommen
        }

        if( last_preset_time != preset_time )                           // hat sich die voreingestellte Zeit gendert?
        {
            eesave_cnt = 90;                                            // ca. 90 Sekunden warten...
            last_preset_time = preset_time;                             // nderung bernommen
        }

        if( eesave_cnt == 1 )                                           // Variable wurde (im Timer-Interrupt) sekndlich herruntergezhlt
        {                                                               // bis sie den Wert 1 enthielt
            if( eeprom_read_word( &ee_time[ i ] ) != preset_time )      // EEprom-Schreiben bei Datengleichheit verhindern
            {                                                           // alte Speicherstelle markieren
                eeprom_write_word( &ee_time[ i ], ( eeprom_read_word( &ee_time[ i ] ) | 0x8000 ) );

                eeprom_write_word( &ee_time[ ++i ], preset_time );      // Voreinstellung speichern
            }

            eesave_cnt = 0;
        }
    }                                                                   // end while( 1 )
}                                                                       // end main()

