#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "main.h"
#include "timer.h"
#include "keys.h"
#include "Led_Display.h"

#ifdef SEGMENTDRIVER
unsigned char PROGMEM conv_tab[] = { 111, 34, 87, 118, 58, 124, 125, 38, 127, 126 }; //Decoder fr Segmente
#else
unsigned char PROGMEM conv_tab[] = { 32, 107, 68, 66, 11, 18, 16, 99, 0, 2 }; //Decoder fr Segmente
#endif

/************************************************************************/
/*			Interrupt Service Routine			*/
/************************************************************************/
ISR( TIMER1_COMPA_vect )
{
    static unsigned int alarm = 0;
    static unsigned char mask = DIGIT1;

#if F_CPU % DEBOUNCE
    OCR1A = F_CPU / DEBOUNCE - 1;		                                // compare DEBOUNCE - 1 times
#endif

    if( --prescaler == 0 )                                              // exact one second over
    {
        prescaler = DEBOUNCE;
        if( eesave_cnt > 1 ) eesave_cnt--;

#if F_CPU % DEBOUNCE			                                        // handle remainder
        OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1;                // compare once per second
#endif

    }                                                                   // end if( --prescaler == 0 )

    if( prescaler == compare )                                          // exakt eine Sekunde (nach Setzen der Variable 'compare') vergangen
    {
        if( time && count_down ) time--;

        else
        {
            IO_PORT |= OUTPUT;                                          // Licht aus (low-aktiv)
            compare = 0;

            if( !time )
            {
                alarm = DEBOUNCE * 4;                                   // Alarm 4 Sekunden lang
                count_down = 0;                                         // stoppe Countdown
            }
        }
    }

    if( alarm )                                                         // ist 'alarm' mehr als Null
    {
        if( !( --alarm % 64 ) )                                         // jeder 64. Interrupt (16x pro Sekunde), 'alarm' wird pre-dekrementiert
        {
            if( alarm & (DEBOUNCE / 2) ) IO_PORT ^= SUMMER;             // alle halbe Sekunde Port-"togglen" erlauben

		    else IO_PORT |= SUMMER;                                     // oder Summer aus (low-aktiv)
        }
    }

    else IO_PORT |= SUMMER;

    if( beeptimer )                                                     // ist 'beeptimer' mehr als Null
    {
        if( --beeptimer ) IO_PORT &= ~SUMMER;
		else IO_PORT |= SUMMER;
    }

    if( ( mask >>= 1 ) < DIGIT4 ) mask = DIGIT1;

    DIGIT_PORT |= ALL_DIGITS;                                           // Digits aus (low-aktiv), also betreffende Pin's high

    switch( mask )                                                      // welches Segment ist dran?
    {
        case DIGIT1: DISP_PORT = Convert_Seg( sekunden_einer ); break;

        case DIGIT2: DISP_PORT = Convert_Seg( sekunden_zehner ); break;

        case DIGIT3: DISP_PORT = Convert_Seg( minuten_einer ); break;

        case DIGIT4:
        {
            if( minuten_zehner ) DISP_PORT = Convert_Seg( minuten_zehner );

            else                                                        // fhrende Null ausblenden
            {

#ifdef SEGMENTDRIVER
                DISP_PORT &= ~ALL_SEGMENTS;                             // Display aus
#else
                DISP_PORT |= ALL_SEGMENTS;                              // Display aus
#endif

            }

            break;
        }

        default: break;
    }

    DIGIT_PORT ^= mask;                                                 // neues Digit ein

    if( !( prescaler % 8 ) )                                            // jeden 8. Interrupt...
	{
        static uint8_t ct0, ct1;
        static uint16_t rpt;
        uint8_t i;

        i = key_state ^ ~KEY_PIN;                                       // key changed ?

        ct0 = ~( ct0 & i );                                             // reset or count ct0
        ct1 = ct0 ^ (ct1 & i);                                          // reset or count ct1
        i &= ct0 & ct1;                                                 // count until roll over ?
        key_state ^= i;                                                 // then toggle debounced state

        if( key_state & KEY2 )                                          // Verriegelung KEY2
        {
            key_press = key_state & i;                                  // 0->1: key press detect, nicht "speichern"
            key_rpt &= ~REPEAT_MASK;                                    // key_repeat lschen
        }

        else key_press |= key_state & i;                                // 0->1: key press detect, "speichern"

        if( (key_state & REPEAT_MASK) == 0 )                            // check repeat function
        {
            rpt = REPEAT_START;                                         // start delay
        }

        if( --rpt == 0 )
        {
            rpt = REPEAT_NEXT;                                          // repeat delay
            key_rpt |= key_state & REPEAT_MASK;
        }
	}
}                                                                       // end ISR


void timer_init ( void )
{
    TCCR1B = (1 << WGM12) ^ (1 << CS10);	                            // CTC mode
                				        	                            // clear on compare
    OCR1A = F_CPU / DEBOUNCE - 1;
    TCNT1 = 0;
    prescaler = DEBOUNCE;

    TIMSK = (1 << OCIE1A);
}
